#ifndef GO_BOARD_H
#define GO_BOARD_H

#include <list>

class XY
{
	public:
		int x, y;
		
		XY( int _x, int _y ): x(_x), y(_y) {}
		XY( const XY & xy ): x(xy.x), y(xy.y) {}
		
		bool operator < ( const XY & xy ) const
		{
			return x < xy.x || y < xy.y;
		}
		
		bool operator == ( const XY & xy ) const
		{
			return x == xy.x && y == xy.y;
		}
		
		const XY & operator = ( const XY & xy )
		{
			x = xy.x;
			y = xy.y;
			
			return *this;
		}
		
};

class Go;

typedef void (*go_callback)( Go *, int, XY, void * );

class Go
{
	public:
		Go( int width, int height );
		~Go();
		
		enum Stone {
			None,
			White,
			Black,
			OffBoard
		};
		
		void UpdateValidPlacements( Stone );
		bool ValidPlacement( int x, int y );
		
		bool PlaceStone( Stone color, int x, int y, bool notify_listeners=true );
		
		Stone StoneAt( int x, int y ) const;
		Stone StoneAt( const XY & xy ) const;
		
		int GetWidth() const;
		int GetHeight() const;
		
		/*
			Get the current score for white or black player
		*/
		int WhiteScore();
		int BlackScore();
		
		/*
		*/
		void AddListener( go_callback, void * );
		
	protected:
		friend class GoNeuralNet;
		
		void GetConnected( int, int, list<XY> & );
		bool ConnectedToFree( list<XY> & );
		bool ConnectedToFree( int, int );
		void SetToNone( list<XY> & );
		void ForceStone( Stone, XY );
		void NotifyListeners( Stone, XY );
		bool CalcValidPlacement( Stone, int, int );
		void UpdateScore();
		
		int fWidth, fHeight;
		int fWhiteScore, fBlackScore;
		
		Stone * fBoard, * fTempBoard, * fTemp2Board;
		
		//list<unsigned char*> fOldStates;
		list<Stone*> fOldStates;
		
		bool * fValidPlacements;
		
		list< pair<go_callback,void*> >	fCallbacks;
		
		bool fPrintedError;
};

#endif
