#include "GoView.h"

#include <stdio.h>
#include <Window.h>
#include <Messenger.h>

//#include "besupport/BThread.h"

//using namespace muscle;

void
update_go_display( Go * go, int color, XY xy, void * _view )
{
	printf("update_go_display\n");
	
	((GoView*)_view)->GoEvent(color,xy);
}

GoView::GoView( BRect rect, Go * go )
:	BView( rect, "Go View", B_FOLLOW_ALL, B_WILL_DRAW|B_FRAME_EVENTS ),
	fGo( go ),
	fColor( Go::White ),
	fEnabled(true),
	fLastStone(-1,-1),
	fSuggestion(-1,-1)
{
	SetViewColor( B_TRANSPARENT_COLOR );

	fGo->UpdateValidPlacements( fColor );
	
	fGo->AddListener( update_go_display, this );
}

GoView::~GoView()
{
}

void
GoView::Draw( BRect rect )
{
	rgb_color bg_col = (rgb_color){218,218,218};
	rgb_color wh_col = (rgb_color){255,255,255};
	rgb_color bl_col = (rgb_color){0,0,0};
	rgb_color re_col = (rgb_color){255,0,0};
	rgb_color bu_col = (rgb_color){0,0,255};
	
	if ( !fEnabled )
	{
		wh_col = (rgb_color){235,235,235};
		bl_col = (rgb_color){188,188,188};
	}
	
	SetHighColor( bg_col );
	FillRect( Bounds() );
	
	float size_x = Bounds().Width() / (float)fGo->GetWidth();
	float size_y = Bounds().Height() / (float)fGo->GetHeight();
	
	BRect outer(0,0, size_x-1, size_y-1);
	BRect inner(outer);
	inner.InsetBy(1,1);
	
	SetHighColor( bl_col );
	
	for ( int y=0; y<fGo->GetHeight(); y++ )
		StrokeLine( BPoint(0,size_y*(y+0.5)), BPoint(Bounds().right, size_y*(y+0.5)) );
	
	for ( int x=0; x<fGo->GetWidth(); x++ )
		StrokeLine( BPoint(size_x*(x+0.5),0), BPoint(size_x*(x+0.5), Bounds().bottom) );
	
	for ( int y=0; y<fGo->GetHeight(); y++ )
		for ( int x=0; x<fGo->GetWidth(); x++ )
		{
			inner.OffsetTo(x*size_x+1, y*size_y+1);
			
			switch ( fGo->StoneAt(x,y) )
			{
				case Go::White:
					SetHighColor( wh_col );
					FillEllipse( inner );
					break;
				case Go::Black:
					SetHighColor( bl_col );
					FillEllipse( inner );
					break;
				default:
					break;
			}
			
			if ( fLastStone == XY(x,y) )
			{
				SetHighColor( re_col );
				StrokeEllipse( inner );
			}

			if ( fSuggestion == XY(x,y) )
			{
				SetHighColor( bu_col );
				StrokeEllipse( inner );
			}
		}
}

void
GoView::MouseDown( BPoint where )
{
	int32 buttons = 0;
	
	Window()->CurrentMessage()->FindInt32("buttons", &buttons );
	
	float size_x = Bounds().Width() / fGo->GetWidth();
	float size_y = Bounds().Height() / fGo->GetHeight();
	
	int x = (int)(where.x / size_x);
	int y = (int)(where.y / size_y);
	
	if ( buttons & B_PRIMARY_MOUSE_BUTTON )
	{
		if ( !fEnabled )
			return;
	
		if ( fGo->ValidPlacement( x, y ) )
		{
			fGo->PlaceStone( fColor, x, y );
		
			Invalidate();
		} else
		{
			printf("Invalid placement\n");
		}
	}

	if ( buttons & B_SECONDARY_MOUSE_BUTTON )
	{
		BMessage msg('Sugg');
		msg.AddInt32("x", x);
		msg.AddInt32("y", y);
		
		fSuggestion = XY(x,y);
		Invalidate();
		
		BMessenger msgr(Window());
		msgr.SendMessage( &msg );
	}
}

void
GoView::FrameResized( float, float )
{
	Invalidate();
}

void
GoView::MessageReceived( BMessage * msg )
{
	switch ( msg->what )
	{
		default:
			BView::MessageReceived(msg);
			break;
	}
}

void
GoView::SetEnabled( bool enabled )
{
	fEnabled = enabled;
	
	Invalidate();
}

void
GoView::GoEvent( int color, XY xy )
{
	fLastStone = xy;
	Invalidate();
}

void
GoView::SetSuggestion( XY xy )
{
	fSuggestion = xy;
	
	Invalidate();
}
